package othello.view;

import java.awt.*;
import java.awt.geom.AffineTransform ;
import java.util.*;
import javax.swing.JPanel;

import othello.model.GameConstants;

/**
 * <p>Title: Othello</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Memorial University</p>
 * @author Theodore Norvell
 * @version 1.0
 */

public class BoardView extends JPanel implements Observer, GameConstants {

    private static final long serialVersionUID = 1L;
	Animator animationModel ;
    
    public BoardView(Animator animationModel ) {
        this.animationModel = animationModel ;
        animationModel.addObserver(this) ; }
        
    /** This implements the Observer interface
     */
    public void update(Observable o, Object arg) {
        repaint() ; }
        
    
    @Override protected void paintComponent( Graphics g ) {
    	super.paintComponent( g ) ;
        Point p0, p1 ;
        Color originalColor = g.getColor() ;
        /* Paint the green background for the board. */ {
        	g.setColor(Color.WHITE) ;
        	g.fillRect( 0, 0, getWidth(), getHeight() ) ;
            p0 = worldToView( 0.0, 0.0 ) ;
            p1 = worldToView( (double)ROWS, (double)COLS ) ;
            g.setColor(new Color(0, 200, 0) ) ;
            g.fillRect(p0.x, p0.y, p1.x-p0.x, p1.y-p0.y ); }
            
        /* Draw vertical lines */ {
            g.setColor(Color.black);
            for( int c=0; c < COLS+1 ; ++c ) {
                p0 = worldToView( 0.0, (float)c ) ;
                p1 = worldToView( (float)COLS, (float)c ) ;
                g.drawLine(p0.x, p0.y, p1.x, p1.y) ; } }
                
        /* Draw horizontal lines */ {
            g.setColor(Color.black);
            for( int r=0; r < ROWS+1 ; ++r ) {
                p0 = worldToView( (float)r, 0.0 ) ;
                p1 = worldToView( (float)r, (float)COLS ) ;
                g.drawLine(p0.x, p0.y, p1.x, p1.y) ; } }
        
        /* Draw the static pieces */ {
            for( int r=0 ; r < ROWS ; ++r ) {
                for( int c=0 ; c < COLS ; ++c ) {
                    int colour = animationModel.getPieceAt(r,c) ;
                    if( colour != EMPTY ) {
                        p0 = worldToView( r+0.1, c+0.1 ) ;
                        p1 = worldToView( r+0.9, c+0.9 ) ;
                        paintPiece( g, colour, p0, p1 ) ; } } } }
        
        /* Draw any flipping piece */ {
            if( animationModel.getState() == animationModel.FLIPPING_STATE ) {
                int colour = animationModel.getNewColour() ;
                int r = animationModel.getFlippingRow() ;
                int c = animationModel.getFlippingCol() ;
                double amount = animationModel.getFlippingAmount() ;
                if( amount < 0.5 ) {
                    colour = colour==BLACK ? WHITE : BLACK ; }
                else { 
                    amount = 1 - amount ; }
                p0 = worldToView( r+0.1+amount*0.8, c+0.1 ) ;
                p1 = worldToView( r+0.9-amount*0.8, c+0.9 ) ;
                paintPiece( g, colour, p0, p1 ) ; } }
        
        /* Draw any flying piece */ {
            if( animationModel.getState() == animationModel.FLYING_STATE ) {
                int colour = animationModel.getNewColour() ;
                double r = animationModel.getFlyingRow() ;
                double c = animationModel.getFlyingCol() ;
                p0 = worldToView( r+0.1, c+0.1 ) ;
                p1 = worldToView( r+0.9, c+0.9 ) ;
                paintPiece( g, colour, p0, p1 ) ; } }
        
        /* Draw the title */ {
        	g.setColor( Color.BLACK ) ;
            paintString( g, -1.9, 0.0, -0.1, COLS, "OTHELLO" ) ; }
        
        /* Put everything back */ {
        	g.setColor( originalColor ) ;
        }
    }
                
            
            
    private void paintPiece( Graphics g, int colour, Point p0, Point p1 ) {
        // Draw a piece
            int width = p1.x-p0.x ;
            int height = p1.y-p0.y ;
            g.setColor( colour==BLACK ? Color.black  : Color.white ) ;
            g.fillOval(p0.x, p0.y, width, height);
            g.setColor( colour==BLACK ? Color.white  : Color.black );
            g.drawOval(p0.x, p0.y, width, height); }
    
    private Point worldToView( double row, double col ) {
        double margin = 2.0 ;
        int height = getHeight() ;
        double vertStretch = height / (ROWS+2*margin) ;
        int y = (int)( (margin+row)*vertStretch ) ;
        int width = getWidth() ;
        double horStretch = width / (COLS+2*margin) ;
        int x = (int)( (margin+col)*horStretch ) ;
        return new Point( x, y ) ; }
    
    private void paintString( Graphics g, double rTop, double cLeft,
                                          double rBot, double cRight,
                                          String message  ) {
        Point p0 = worldToView( rTop, cLeft ) ;
        Point p1 = worldToView( rBot, cRight ) ;
        int width = p1.x - p0.x ;
        int height = p1.y - p0.y ;
        // Start with a font
            Font font = new Font("SansSerif", Font.PLAIN, 10 ) ;
        // Calculate the height and width of the string in this font
            FontMetrics fm = g.getFontMetrics(font) ;
            int h = fm.getMaxAscent() ;
            int w = fm.stringWidth(message) ;
        // Calculate the scale factor to make it fit in one dimension.
            double scaleFactor = Math.min( (double)height / h, (double)width / w) ;
        // Create an Affine transform that will scale the font.
            AffineTransform xform = (new AffineTransform()) ;
            xform.scale( scaleFactor, scaleFactor);
        // Create a new font
            font = font.deriveFont(xform) ;
        
        // Centre the message left to right.
            int messWidth = g.getFontMetrics(font).stringWidth(message) ;
            int x = p0.x+ (width-messWidth)/2 ;
            int y = p1.y ;
        // Put it on the screen
        g.setFont( font );
        g.drawString(message, x, y ) ; }

}